GLOBAL.setmetatable(GLOBAL.getfenv(1), {__index = function(self, index)
	return GLOBAL.rawget(GLOBAL, index)
end})

local function overwrite(tabula, name, ante, post)
	if type(tabula) ~= "table" then return end 
	local old = tabula[name]
	tabula[name] = function(...)
		local unpack = unpack or table.unpack or GLOBAL.unpack
		
		local results_ante = {}
		if type(ante) == "function" then results_ante = {ante(...)} end 
		if #results_ante > 0 then return unpack(results_ante) end 
		
		local results_original = {}
		if type(old) == "function" then results_original = {old(...)} end 
		
		local results_post = {}
		if type(post) == "function" then results_post = {post(...)} end 
		if #results_post > 0 then return unpack(results_post) end 
		
		return unpack(results_original)
	end 
end 

TUNING.BLUNDERBUSS_MULTIPLIER = 2
TUNING.BLUNDERBUSS_MAX_WORK = 10-- default # of works in explosive component
-- this multiplier is used in 3 places: 1) scale; 2) speed reduction; 3) range of damage
-- this is multiplied by the default gunpowder range which is 2

local NOTAGS = {"INLIMBO", "NOCLICK", "FX", "shadow", "shadowminion"}
local NOTAGS_EASY = {"abigail", "wall", "companion", "player"}

if false then-- this has got to be as dangerous as an american airstrike 
	for _, tag in pairs(NOTAGS_EASY) do 
		table.insert(NOTAGS, tag) 
	end 
end

local function onhit(inst, attacker, target, weapon)
	local x, y, z = inst.Transform:GetWorldPosition()
	local range = TUNING.GUNPOWDER_RANGE * TUNING.BLUNDERBUSS_MULTIPLIER
	
	local owner = inst.components.projectile.owner 
	if owner and owner.components.inventoryitem and owner.components.inventoryitem.owner then 
		owner = owner.components.inventoryitem.owner
	end 
	
	for _, ent in pairs(TheSim:FindEntities(x, y, z, range, nil, NOTAGS)) do 
-- the on burnt function in the explosive component checks doubling for inlimbo tag and then whether its in pocket? 
		local a, b, c = ent.Transform:GetWorldPosition()
		local dist_sq = (x - a) ^ 2 + (z - c) ^ 2 + (y - b) ^ 2-- y axis included 
		local multi = 1 - dist_sq / range ^ 2
		
		local damage = TUNING.GUNPOWDER_DAMAGE * multi
		local works = TUNING.BLUNDERBUSS_MAX_WORK * multi
		
		local function explode(ent)
			if ent.components.combat and ent.components.combat:GetAttacked(attacker, damage, weapon) then 
				return print("attacked: target, damage, multi:", ent, damage, multi)
			elseif ent.components.workable then 
				ent.components.workable:WorkedBy(attacker, works)
				return print("worked: target, damage, multi:", ent, works, multi)
			end 
		end 
		
		explode(ent)
		
-- the more elegant, concise, but abstract and confusing way to do it: 
-- if ent.components.combat and ent.components.combat:GetAttacked(attacker, damage, weapon) 
-- or ent.components.workable and ent.components.workable:WorkedBy(attacker, works) then end 
	end 
	
	local explosion = SpawnPrefab"explode_small"
	explosion.Transform:SetPosition(x, y, z)
end

AddPrefabPostInit("gunpowder_projectile", function(inst)
	RemovePhysicsColliders(inst)
	
	local scale = TUNING.BLUNDERBUSS_MULTIPLIER
	inst.Transform:SetScale(scale, scale, scale)
	
	local projectile = inst.components.projectile
	projectile.speed = projectile.speed / TUNING.BLUNDERBUSS_MULTIPLIER
	
	overwrite(projectile, "onhit", onhit)
end)

local function imagechange(inst)
	if inst.components.weapon then 
		inst.components.weapon:SetDamage(0)
	end 
end

AddPrefabPostInit("blunderbuss", function(inst)
	inst:ListenForEvent("imagechange", imagechange)
end)
